% Permalloy, Perpendicular FMR

% Calculating the surface impedance of a permalloy film with magnetization
% Perpendicular to the plane. Theta_k = 0.
% CGS units.
clc;
clear all;
close all;

global d V
% Calculate the dispersion relation first.
global w mu0 eps Xi1s Xi2s k1s k2s Z1s Z2s h1r h1l h2r h2l

mu0 = 4*pi*10^(-9); % henry/cm
eps0 = 1/(36*pi)*10^(-11); % F/cm
Z0 = sqrt(mu0/eps0);
sigma = 0.7*10^5; % mho/cm
A = 1.14e-6; % erg/cm
g = 2.10;
M = 10000/(4*pi); % Gauss
Ho = 2500:1:4500; % Oe
% Ho = 2500:1:9000; % Oe
L = length(Ho);

gamma = 2*pi*1.4*10^6*g; % Hz/Oe
w = gamma*3500; % GHz
f = w/(2*pi);
dH = 2*4.33e-3*w/gamma; % Oe
omega=w*(1-j*4.33e-3);
w=omega;
eps = -j*sigma/omega; % F/cm
k0_sq = -j*omega*sigma*mu0;

% Resonant mode.

c1 = ones(1,L);
c2 = -( k0_sq + (w/gamma-Ho)*M/(2*A) );

c3 = M/(2*A)*k0_sq*(w/gamma-Ho-4*pi*M);
k1_sq = -c2./(2*c1) + sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
k2_sq = -c2./(2*c1) - sqrt(c2.^2 - 4*c1.*c3)./(2*c1);

k1 = sqrt(k1_sq);
k1 = sign(-imag(k1)).*k1; % Pick up the physical mode (non-growing).
k2 = sqrt(k2_sq);
k2 = sign(-imag(k2)).*k2;

% % Non-resonant mode.

% c1 = ones(1,L);
% c2 = -( k0_sq + (-w/gamma-Ho).*M/(2*A) );
% c3 = M/(2*A)*k0_sq.*(-w/gamma-Ho-4*pi*M);
% k3_sq = -c2./(2*c1) + sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
% k4_sq = -c2./(2*c1) - sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
%
% k3 = sqrt(k3_sq);
% k3 = sign(-imag(k3)).*k3;
% k4 = sqrt(k4_sq);
% k4 = sign(-imag(k4)).*k4;

%  Mu_r = 1+4*pi*M./(Ho-w/gamma);
%ZV=j.*sqrt(mu0.*Mu_r./eps);%volume char. imp.

figure(1);%k vs H
hold on;

plot(real(k1),Ho,'b','Linewidth',2);
plot(imag(k1),Ho,'b--','Linewidth',2);
plot(real(k2),Ho,'r','Linewidth',2);
plot(imag(k2),Ho,'r--','Linewidth',2);

% plot(Ho,real(k3),'g','Linewidth',2);
% plot(Ho,imag(k3),'g--','Linewidth',2);
% plot(Ho,abs(real(k4)),'cyan','Linewidth',2);
% plot(Ho,imag(k4),'cyan--','Linewidth',2);

ylabel('Ho(Oe)','Fontsize',24);
%xlabel('Im(k) & Re(k) (cm^-^1)','Fontsize', 24);
xlabel('k(rad/cm)','Fontsize', 24);

h = gca;
set(h,'Fontsize',22);
title('Permalloy Dispersion Diagram','FontSize',24);
hold off;


% Calculate the surface impedance, only consider resonant modes

d = 0.3E-4; % cm

Xi1 = 4*pi*M./(Ho+2*A/M.*k1_sq-w/gamma ); %
Xi2 = 4*pi*M./(Ho+2*A/M.*k2_sq-w/gamma ); %
Ks_0 = 0; Ks_d = 0;
n_minus_2pi=0;
n_plus_2pi=0;
indicator=0;
for i = 1:L

    S = [ 1 1 1 1; exp(-j*k1(i)*d) exp(j*k1(i)*d) exp(-j*k2(i)*d) exp(j*k2(i)*d); ...
        Xi1(i)*(Ks_0+j*k1(i)*A) Xi1(i)*(Ks_0-j*k1(i)*A) Xi2(i)*(Ks_0+j*k2(i)*A) Xi2(i)*(Ks_0-j*k2(i)*A); ...
        Xi1(i)*(Ks_d+j*k1(i)*A)*exp(-j*k1(i)*d) Xi1(i)*(Ks_d-j*k1(i)*A)*exp(j*k1(i)*d) Xi2(i)*(Ks_d+j*k2(i)*A)*exp(-j*k2(i)*d) Xi2(i)*(Ks_d-j*k2(i)*A)*exp(j*k2(i)*d)];

    b = [1 1 0 0]'; % boundary condition
    h_vec(:,i) = inv(S)*b; % h_vec(:,i) = [h1+(i) h1-(i) h2+(i) h2-(i)]'

    Z1(i) = k1(i)/(omega*eps);
    Z2(i) = k2(i)/(omega*eps);
    
    %Zs_0(i) = Z1(i)*(h_vec(1,i)-h_vec(2,i)) + Z2(i)*(h_vec(3,i)-h_vec(4,i));
   % Zs_d(i) = Z1(i)*(h_vec(1,i)*exp(-j*k1(i)*d)-h_vec(2,i)*exp(j*k1(i)*d))...
        + Z2(i)*(h_vec(3,i)*exp(-j*k2(i)*d)-h_vec(4,i)*exp(j*k2(i)*d));

    SP = [ j*Z1(i)*exp(-j*k1(i)*d) -j*Z1(i)*exp(j*k1(i)*d) j*Z2(i)*exp(-j*k2(i)*d) -j*Z2(i)*exp(j*k2(i)*d); ...
        exp(-j*k1(i)*d) exp(j*k1(i)*d) exp(-j*k2(i)*d) exp(j*k2(i)*d); ...
        Xi1(i)*(Ks_0+j*k1(i)*A) Xi1(i)*(Ks_0-j*k1(i)*A) Xi2(i)*(Ks_0+j*k2(i)*A) Xi2(i)*(Ks_0-j*k2(i)*A); ...
        Xi1(i)*(Ks_d+j*k1(i)*A)*exp(-j*k1(i)*d) Xi1(i)*(Ks_d-j*k1(i)*A)*exp(j*k1(i)*d)...
        Xi2(i)*(Ks_d+j*k2(i)*A)*exp(-j*k2(i)*d) Xi2(i)*(Ks_d-j*k2(i)*A)*exp(j*k2(i)*d)];
    A_mat=A_matrix(SP, Z1(i), Z2(i));
    a11(i)=A_mat(1, 1);
    D=A_mat(1, 1)*A_mat(2, 2)-A_mat(1, 2)*A_mat(2, 1);
    Z_square=A_mat(1, 2)/A_mat(2, 1);
    Z(i)=sign(imag(Z_square))*sqrt(Z_square);
    % calculation of alpha and beta: keff=beta+jalpha

    ZS(i)=-j*(A_mat(1,1)-1)/A_mat(2,1);
    emcoupl(i)=(j*ZS(i))/Z(i);
    C(i)= A_mat(1,1)+(A_mat(1,2))/Z(i);%1/S21,matched imp.
    %C=exp(jwt)=1/S21(S11=0)
     thet(i)=angle(C(i));
    beta(i)=thet(i)/d;
    alpha(i)=log(abs(C(i)))/d;
    % if beta(i)>0
    %  beta(i)=-2*pi/d+beta(i);
    %  end

%     if real(C(i))<0
%        R(i)=imag(C(i))/real(C(i));
%        beta(i)=-atan(R(i))/d;
%       end
%          if beta(i) < -pi/(2*d)
%              beta(i)=-beta(i)-pi/d;
%         elseif beta(i)> pi/(2*d)
%               beta(i)=-beta(i)+pi/d;
%           end

      keff(i)=beta(i)-j*alpha(i);
    % ZS(i)=Z(i)*tan(keff(i)*(d/2));
    % emcoupl(i)=j*tan(keff(i)*(d/2));
    % ZS(i)=A_mat(1,1);
    %  ZS(i)=cos(keff(i)*d);
    % ZS(i)=(j*Z(i))*sin(keff(i)*d);
    %  ZS(i)=(j/Z(i))*sin(keff(i)*d);
    Mueff(i)=((keff(i))*Z(i))/(w.*mu0);
    Mueff(i)=-j*Mueff(i);
    epseff(i)=(keff(i))/(Z(i)*w);

    epseff(i)=j*epseff(i);
    sigma_eff = epseff.*j.*w;
    Zeff(i)=j*sqrt((Mueff(i)*mu0)./epseff(i));
    Zs_0(i) = Z1(i)*(h_vec(1,i)-h_vec(2,i)) + Z2(i)*(h_vec(3,i)-h_vec(4,i));
    Zs_d(i) = Z1(i)*(h_vec(1,i)*exp(-j*k1(i)*d)-h_vec(2,i)*exp(j*k1(i)*d))...
        + Z2(i)*(h_vec(3,i)*exp(-j*k2(i)*d)-h_vec(4,i)*exp(j*k2(i)*d));

    % Check the result by integrating the poynting vector.
    Xi1s = Xi1(i); Xi2s = Xi2(i); k1s = k1(i); k2s = k2(i); Z1s = Z1(i); Z2s = Z2(i);
    h1r = h_vec(1,i); h1l = h_vec(2,i); h2r = h_vec(3,i); h2l = h_vec(4,i);


    Integ(i) = quadgk(@myfun,0,d);  % 2Zs(i)
    Integ1(i) = quadgk(@myfun1,0,d)/d; % Effective permeability by volume integration
    % Integ2(i) = quadgk(@myfun2,0,d)/(Integ(i)/2)^2/d; % Effective permittivity by volume integration
    %Integ3(i) = quadgk(@myfun3,0,d);
    Integ2(i) = quadgk(@myfun2,0,d)/d;

end

figure(2);%char. impedance,Z, vs H (circ. pol.)
hold on;

plot(Ho,real(Z),'r','Linewidth',2);
plot(Ho,imag(Z),'r--','Linewidth',2);
%plot(Ho,real(ZV),'b','Linewidth',2);
%plot(Ho,imag(ZV),'b--','Linewidth',2);
% plot(Ho, -real(Integ/2),'b','LineWidth',1);
% plot(Ho, -imag(Integ/2),'b--','LineWidth',1);

xlabel('H(Oe)','Fontsize',24);
ylabel('Z(Ohm)','Fontsize', 24);
%ylabel('Im(Z) & Re(Z) (ohms)','Fontsize', 24);
%h = gca;
%set(h,'Fontsize',22);
%hold off;

% axis([2500 4500 -100 100]);
h = gca;
set(h,'Fontsize',22);
title('characteristic impedance of 0.3 \mum Permalloy Film','FontSize',24);
legend('Z Real','Z Imag',2);
% legend('y = 0 Real','y = 0 Imag','y = d Real','y = d Imag','Integ Real', 'Integ Imag',4);
hold off;

figure(3);%effective k vs H (calc. from 2x2 matrix)
hold on;
plot(real(keff),Ho,'r','Linewidth',2);
plot(imag(keff),Ho,'r--','Linewidth',2);
ylabel('H(Oe)','Fontsize',24);
xlabel('keff(rad/cm)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('Propogation constant of 0.3 \mum Permalloy Film','FontSize',24);
legend('real(keff)','imag(keff)',2);
hold off;

figure(4);% surface impedance ,Zs, vs H (from 4x4 matrix)
hold on;

plot(Ho, real(Zs_0),'r','Linewidth',2);
plot(Ho, imag(Zs_0),'r--','Linewidth',2);

% plot(Ho,phi,'r','Linewidth',2);

xlabel('H(Oe)','Fontsize',24);
ylabel('Zs(Ohm)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('Zs_0 of 0.3 \mum Permalloy Film','FontSize',24);
legend('real','Imag',2);
hold off;

figure (5);%surface impedance,ZS,vs H (from 2x2 matrix)
hold on;
plot(Ho,real(ZS),'b','Linewidth',2);
plot(Ho,imag(ZS),'b--','Linewidth',2);

xlabel('H(Oe)','Fontsize',24);
ylabel('ZS(ohm)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('ZS of 0.3 \mum Permalloy Film','FontSize',24);
legend('real','Imag',2);
hold off;

figure(6);%mueff vs H (from poynting vector;mu vs H(from tensor mu)
hold on;
plot(Ho, real(Mueff),'r','LineWidth',1);
plot(Ho, imag(Mueff),'r--', 'LineWidth',1);
plot(Ho, real(Integ1),'b','LineWidth',1);
plot(Ho, imag(Integ1),'b--', 'LineWidth',1);
xlabel('H(Oe)', 'FontSize', 24);
ylabel('Mueff', 'FontSize',24);
%legend('Mueff Real','Mueff Imag','MuS Real','MuS Imag',1);
h = gca;
set(h,'Fontsize',22);
title('Permeability Comparison','FontSize',24);
hold off;
figure(7);%epseff vs H
hold on;
plot(Ho, real(epseff),'b','LineWidth',2);
plot(Ho, imag(epseff),'b--', 'LineWidth',2);
xlabel('H(Oe)', 'FontSize', 24);
ylabel('\epsilon_e_f_f(f/cm)', 'FontSize',32);
legend('\epsilon_e_f_f Real','\epsilon_e_f_f Imag',4);
h = gca;
set(h,'Fontsize',30);
title('Effective Average Permittivity','FontSize',24);
hold off;

figure(8);%sigmaeff vs H (see fig. 7)
hold on;
plot(Ho, real(sigma_eff),'b','LineWidth',2);
plot(Ho, imag(sigma_eff),'r--','LineWidth',2);
plot(Ho, sigma,'r','LineWidth',2);
h = gca;
set(h,'Fontsize',30);
hold off;
xlabel('H(Oe)', 'FontSize', 30);
ylabel('sigma_e_f_f(mhos/cm)', 'FontSize',32);
title('Effective conductivity','FontSize',30);
legend('sigma_e_f_f Real','sigma_e_f_f Imag',4);

figure(9);%Zeff vs H (calculated from sqrt(mu0*mueff/epseff)
hold on;
plot(Ho, real(Zeff),'LineWidth',2);
plot(Ho, imag(Zeff),'r','LineWidth',2);
h = gca;
set(h,'Fontsize',30);
hold off;
xlabel('H(Oe)', 'FontSize', 30);
ylabel('Z_e_f_f(Ohm)', 'FontSize',32);
title('Z_e_f_f','FontSize',30);
legend('Z_e_f_f Real','Z_e_f_f Imag',4);

ZSM2=(abs(Zs_0)).^2;
MuS=mu0*Integ1;

%epsS=-2.*Zs_0./(j.*w.*ZSM2.*d)+MuS./ZSM2;
epsS=Integ2./ZSM2;
%epsS=conj(epsS);
sigmaS = epsS.*j.*w;
Zsv=0.5.*(j.*w.*d.*(MuS-(conj(epsS).*ZSM2)));% also, Zs


figure(10);%epsS vs H (from poynting vector)
hold on;
plot(Ho, real(epsS),'b','LineWidth',2);
plot(Ho, imag(epsS),'b--', 'LineWidth',2);
xlabel('H(Oe)', 'FontSize', 24);
ylabel('epsS(f/cm)', 'FontSize',32);
legend('epsS Real','epsS Imag',4);
h = gca;
set(h,'Fontsize',30);
title('Surface Average Permittivity','FontSize',24);
hold off;
figure(11);%sigmaS vs H (see fig. 7)
hold on;
plot(Ho, real(sigmaS),'b','LineWidth',2);
plot(Ho, imag(sigmaS),'r--','LineWidth',2);
plot(Ho, sigmaS,'r','LineWidth',2);
h = gca;
set(h,'Fontsize',30);
hold off;
xlabel('H(Oe)', 'FontSize', 30);
ylabel('\sigmaS(mhos/cm)', 'FontSize',32);
title('Surface conductivity','FontSize',30);
legend('\sigmaS Real','\sigmaS Imag',4);

figure(12);%Zsv vs H (calculated from poynting integral)
hold on;
%plot(Ho, real(Zsv),'LineWidth',2);
%plot(Ho, imag(Zsv),'r','LineWidth',2);
plot(Ho, real(Integ1),'LineWidth',2);
plot(Ho, -imag(Integ1),'r','LineWidth',2);
h = gca;
set(h,'Fontsize',30);
hold off;
xlabel('H(Oe)', 'FontSize', 30);
ylabel('MuS', 'FontSize',32);
title('MuS','FontSize',30);
legend('MuS Real','MuS Imag',4);

figure(13);%emcoupl(i)
hold on;
plot(Ho, real(emcoupl),'b','LineWidth',2);
plot(Ho, imag(emcoupl),'b--', 'LineWidth',2);
xlabel('H(Oe)', 'FontSize', 24);
ylabel('emcoupl)', 'FontSize',32);
legend('emcoupl real','emcoupl Imag',4);
h = gca;
set(h,'Fontsize',30);
title('em coupling','FontSize',24);
hold off;


% Plot out the h(y) at Ho = 2500 Oe.

N = 501;

y = 0:d/200:d;
hy = h_vec(1,N)*exp(-j*k1(N)*y) + h_vec(2,N)*exp(j*k1(N)*y) + h_vec(3,N).*exp(-j*k2(N)*y) + h_vec(4,N).*exp(j*k2(N)*y);

% figure(3);
% hold on;
% plot(y,real(hy),'LineWidth',2);
% % plot(y,imag(hy),'g','LineWidth',2);
% % plot(y,abs(hy),'r','LineWidth',2);
% h = gca;
% set(h,'Fontsize',30);
% % axis([0 d 0.8 1]);
% xlabel('y(cm)','Fontsize',30);
% ylabel('Re(h(y))','Fontsize',30);
% title(['Ho = ', num2str(Ho(N)), 'Oe'],'Fontsize',30);
% hold off;

%X. Save raw data file for figure 1%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Plot out the h(y) at Ho = 2500 Oe.

N = 501;

y = 0:d/200:d;
hy = h_vec(1,N)*exp(-j*k1(N)*y) + h_vec(2,N)*exp(j*k1(N)*y) + h_vec(3,N).*exp(-j*k2(N)*y) + h_vec(4,N).*exp(j*k2(N)*y);

% figure(3);
% hold on;
% plot(y,real(hy),'LineWidth',2);
% % plot(y,imag(hy),'g','LineWidth',2);
% % plot(y,abs(hy),'r','LineWidth',2);
% h = gca;
% set(h,'Fontsize',30);
% % axis([0 d 0.8 1]);
% xlabel('y(cm)','Fontsize',30);
% ylabel('Re(h(y))','Fontsize',30);
% title(['Ho = ', num2str(Ho(N)), 'Oe'],'Fontsize',30);
% hold off;
% 

