% Calculating the surface impedance of a permalloy film with magnetization 
% Perpendicular to the plane. Theta_k = 0. 
% CGS units.
clc;
clear all;
close all;

global d V
% Calculate the dispersion relation first. 
global w mu0 eps Xi1s Xi2s k1s k2s Z1s Z2s h1r h1l h2r h2l

mu0 = 4*pi*10^(-9); % henry/cm
eps0 = 1/(36*pi)*10^(-11); % F/cm
Z0 = sqrt(mu0/eps0);
sigma = 0.7*10^5; % mho/cm
A = 1.14e-6; % erg/cm
g = 2.10;
M = 10000/(4*pi); % Gauss
Ho = 2500:1:4500; % Oe
% Ho = 2500:1:9000; % Oe
L = length(Ho);

gamma = 2*pi*1.4*10^6*g; % Hz/Oe
w = gamma*3500; % GHz
f = w/(2*pi);
dH = 2*4.33e-3*w/gamma; % Oe
eps = -j*sigma/w; % F/cm
k0_sq = -j*w*sigma*mu0;

% Resonant mode.

c1 = ones(1,L);
c2 = -( k0_sq + (w/gamma-i*dH/2-Ho)*M/(2*A) );
c3 = M/(2*A)*k0_sq*(w/gamma-i*dH/2-Ho-4*pi*M);

k1_sq = -c2./(2*c1) + sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
k2_sq = -c2./(2*c1) - sqrt(c2.^2 - 4*c1.*c3)./(2*c1);

k1 = sqrt(k1_sq);
k1 = sign(-imag(k1)).*k1; % Pick up the physical mode (non-growing).
k2 = sqrt(k2_sq);
k2 = sign(-imag(k2)).*k2;

% % Non-resonant mode.

% c1 = ones(1,L);
% c2 = -( k0_sq + (-w/gamma+i*dH/2-Ho).*M/(2*A) );
% c3 = M/(2*A)*k0_sq.*(-w/gamma+i*dH/2-Ho-4*pi*M);
% k3_sq = -c2./(2*c1) + sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
% k4_sq = -c2./(2*c1) - sqrt(c2.^2 - 4*c1.*c3)./(2*c1);
% 
% k3 = sqrt(k3_sq);
% k3 = sign(-imag(k3)).*k3;
% k4 = sqrt(k4_sq);
% k4 = sign(-imag(k4)).*k4;

figure(1);
hold on;

plot(Ho,real(k1),'b','Linewidth',2);
plot(Ho,imag(k1),'b--','Linewidth',2);
plot(Ho,real(k2),'r','Linewidth',2);
plot(Ho,imag(k2),'r--','Linewidth',2);

% plot(Ho,real(k3),'g','Linewidth',2);
% plot(Ho,imag(k3),'g--','Linewidth',2);
% plot(Ho,abs(real(k4)),'cyan','Linewidth',2);
% plot(Ho,imag(k4),'cyan--','Linewidth',2);

xlabel('Ho(Oe)','Fontsize',24);
ylabel('Im(k) & Re(k) (cm^-^1)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('Permalloy Dispersion Diagram','FontSize',24);
hold off;


% Calculate the surface impedance, only consider resonant modes

d = 0.3E-4; % cm

Xi1 = 4*pi*M./(Ho+2*A/M.*k1_sq-w/gamma + j*dH/2); %
Xi2 = 4*pi*M./(Ho+2*A/M.*k2_sq-w/gamma + j*dH/2); %
Ks_0 = 0; Ks_d = 0;
n_minus_2pi=0;
n_plus_2pi=0;
for i = 1:L

%     S = [ 1 1 1 1; exp(-j*k1(i)*d) exp(j*k1(i)*d) exp(-j*k2(i)*d) exp(j*k2(i)*d); ...
%         Xi1(i)*(Ks_0+j*k1(i)*A) Xi1(i)*(Ks_0-j*k1(i)*A) Xi2(i)*(Ks_0+j*k2(i)*A) Xi2(i)*(Ks_0-j*k2(i)*A); ...
%         Xi1(i)*(Ks_d+j*k1(i)*A)*exp(-j*k1(i)*d) Xi1(i)*(Ks_d-j*k1(i)*A)*exp(j*k1(i)*d) Xi2(i)*(Ks_d+j*k2(i)*A)*exp(-j*k2(i)*d) Xi2(i)*(Ks_d-j*k2(i)*A)*exp(j*k2(i)*d)];

    b = [1 1 0 0]'; % boundary condition
%     h_vec(:,i) = inv(S)*b; % h_vec(:,i) = [h1+(i) h1-(i) h2+(i) h2-(i)]'
    
    % h_vec(:,i) = S\b;
    Z1(i) = k1(i)/(w*eps);
    Z2(i) = k2(i)/(w*eps);
    
    SP = [ j*Z1(i)*exp(-j*k1(i)*d) -j*Z1(i)*exp(j*k1(i)*d) j*Z2(i)*exp(-j*k2(i)*d) -j*Z2(i)*exp(j*k2(i)*d); ...
        exp(-j*k1(i)*d) exp(j*k1(i)*d) exp(-j*k2(i)*d) exp(j*k2(i)*d); ...
        Xi1(i)*(Ks_0+j*k1(i)*A) Xi1(i)*(Ks_0-j*k1(i)*A) Xi2(i)*(Ks_0+j*k2(i)*A) Xi2(i)*(Ks_0-j*k2(i)*A); ...
        Xi1(i)*(Ks_d+j*k1(i)*A)*exp(-j*k1(i)*d) Xi1(i)*(Ks_d-j*k1(i)*A)*exp(j*k1(i)*d)...
        Xi2(i)*(Ks_d+j*k2(i)*A)*exp(-j*k2(i)*d) Xi2(i)*(Ks_d-j*k2(i)*A)*exp(j*k2(i)*d)];
    A_mat=A_matrix(SP, Z1(i), Z2(i));
    a11(i)=A_mat(1, 1);
    D=A_mat(1, 1)*A_mat(2, 2)-A_mat(1, 2)*A_mat(2, 1);
    Z_square=A_mat(1, 2)/A_mat(2, 1);
    Z(i)=sign(imag(Z_square))*sqrt(Z_square);
    
%     Zs_new(i)=(A_mat(1, 1)*Z(i)+A_mat(1, 2))/(A_mat(2, 1)*Z(i)+A_mat(2, 2));
    C=A_mat(1, 2)/(Z(i)*A_mat(1, 1));
    V=abs((1+C)/(1-C));
%    V=A_mat(1,1);
%    V=(1+C)/(1-C);
%    x0=2*10^(+5)-j*.2*10^(+5);
%    x=fsolve(@myfun, x0);
%    alpha(i)=real(x);
%    beta(i)=imag(x);
     phi_wrapped(i)=angle((1+C)/(1-C));
     if i>1
       if phi_wrapped(i)-phi_wrapped(i-1)>1.5*pi
           n_minus_2pi=n_minus_2pi+1;
        elseif phi_wrapped(i)-phi_wrapped(i-1)<-1.5*pi
            n_plus_2pi=n_plus_2pi+1;
        end
        
   end
    phi(i)=phi_wrapped(i)-n_minus_2pi*2*pi+n_plus_2pi*2*pi;
%     thet=angle((1+C)/(1-C));
%     phi(i)=unwrap(thet);
     alpha(i)=log(V)/(2*d);
     beta(i)=phi(i)/(2*d);
    
    Zs_0(i)=Z(i)*tan((beta(i)-j*alpha(i))*(d/2));
    
%     Zs_0(i) = Z1(i)*(h_vec(1,i)-h_vec(2,i)) + Z2(i)*(h_vec(3,i)-h_vec(4,i));
%     Zs_d(i) = Z1(i)*(h_vec(1,i)*exp(-j*k1(i)*d)-h_vec(2,i)*exp(j*k1(i)*d)) + Z2(i)*(h_vec(3,i)*exp(-j*k2(i)*d)-h_vec(4,i)*exp(j*k2(i)*d));
%     
%     % Check the result by integrating the poynting vector.
%     Xi1s = Xi1(i); Xi2s = Xi2(i); k1s = k1(i); k2s = k2(i); Z1s = Z1(i); Z2s = Z2(i);
%     h1r = h_vec(1,i); h1l = h_vec(2,i); h2r = h_vec(3,i); h2l = h_vec(4,i); 
%     
% 
%     Integ(i) = quadgk(@myfun,0,d);  % 2Zs(i)
   
end

figure(2);
hold on;
 
% plot(Ho,-real(Zs_d),'r','Linewidth',2);
% plot(Ho,-imag(Zs_d),'r--','Linewidth',2);

plot(Ho,real(Z),'r','Linewidth',2);
plot(Ho,imag(Z),'r--','Linewidth',2);

% plot(Ho, -real(Integ/2),'b','LineWidth',1);
% plot(Ho, -imag(Integ/2),'b--','LineWidth',1);

xlabel('H(Oe)','Fontsize',24);
ylabel('Z(Ohm)','Fontsize', 24);

% axis([2500 4500 -100 100]);
h = gca;
set(h,'Fontsize',22);
title('characteristic impedance of 0.3 \mum Permalloy Film','FontSize',24);
legend('Z Real','Z Imag',2);
% legend('y = 0 Real','y = 0 Imag','y = d Real','y = d Imag','Integ Real', 'Integ Imag',4);
hold off;

figure(3)
hold on;
plot(Ho,beta,'r','Linewidth',2);
plot(Ho,alpha,'r--','Linewidth',2);
xlabel('H(Oe)','Fontsize',24);
ylabel('k(rad/m)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('Propogation constant of 0.3 \mum Permalloy Film','FontSize',24);
legend('beta','alpha',2);
hold off;

figure(4)
hold on;
plot(Ho,phi,'r','Linewidth',2);

xlabel('H(Oe)','Fontsize',24);
ylabel('phi(rad)','Fontsize', 24);
h = gca;
set(h,'Fontsize',22);
title('phi of 0.3 \mum Permalloy Film','FontSize',24);
legend('beta','alpha',2);
hold off;

figure (5)
hold on;
 plot(Ho,real(Zs_0),'b','Linewidth',2);
 plot(Ho,imag(Zs_0),'b--','Linewidth',2);
hold off;
% % Plot out the h(y) at Ho = 2500 Oe. 
% 
% N = 501;
% 
% y = 0:d/200:d;
% hy = h_vec(1,N)*exp(-j*k1(N)*y) + h_vec(2,N)*exp(j*k1(N)*y) + h_vec(3,N).*exp(-j*k2(N)*y) + h_vec(4,N).*exp(j*k2(N)*y);
% 
% figure(3);
% hold on;
% plot(y,real(hy),'LineWidth',2);
% % plot(y,imag(hy),'g','LineWidth',2);
% % plot(y,abs(hy),'r','LineWidth',2);
% h = gca;
% set(h,'Fontsize',30);
% % axis([0 d 0.8 1]);
% xlabel('y(cm)','Fontsize',30);
% ylabel('Re(h(y))','Fontsize',30);
% title(['Ho = ', num2str(Ho(N)), 'Oe'],'Fontsize',30);
% hold off;


